<?php

namespace Drupal\currencyapi\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * A class for currency output.
 */
class CurrencyApiController extends ControllerBase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new CurrencyApiService.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Fetch data from currencyapi.com.
   */
  public function fetchRates(): array {
    return ['#markup' => $this->t("Fetch is made Cron")];
  }

  /**
   * Currency Exchange Title.
   */
  public function currencyExchangeTitle(): string {
    $title = "Currency Exchange";
    return $title;
  }

  /**
   * Info page.
   */
  public function currencyExchange(): array {

    $d = [];
    $d['service'] = \Drupal::service('currencyapi.service');
    $d['currency']['base']['code'] = 'EUR';
    $d['currency']['base']['amount'] = 100;
    $d['config'] = $this->config('currencyapi.settings');

    $d['storage'] = $d['service']->getAllRates();

    foreach ($d['storage'] as $code => $value) {

      if ($value['is_default'] != TRUE) {
        $d['list'][$value['code']] = $value;
        $d['list'][$code]['buy'] = (float) $d['config']->get('buyrate_' . $code);

        $d['list'][$code]['sell'] = $d['service']
          ->convert($d['currency']['base']['amount'] + ($d['currency']['base']['amount'] / 100 * $d['list'][$code]['buy']), 'EUR', $code);
        // Optional, ceil to integer.
        // $d['list'][$code]['sell'] = $d['service']
        // ->ceil($d['list'][$code]['sell']);.
      }
    }

    return [
      '#theme' => 'currency_exchange',
      '#rates' => $d['storage'],
      '#base_currency' => $d['currency']['base']['code'],
      '#list' => $d['list'],
      '#amount' => $d['currency']['base']['amount'],
      '#attached' => [
        'library' => ['currencyapi/currency_rates'],
      ],
    ];
  }

  /**
   * Info page.
   */
  public function ratesPage(): array {

    $d = [];
    $d['service'] = \Drupal::service('currencyapi.service');
    $d['currency']['base']['code'] = 'EUR';
    $d['currency']['base']['amount'] = 100;

    $d['storage'] = $d['service']->getAllRates();

    foreach ($d['storage'] as $code => $value) {
      if ($value['is_default'] == TRUE) {
        $d['base_currency'] = $code;
      }
    }

    return [
      '#theme' => 'currency_rates',
      '#rates' => $d['storage'],
      '#base_currency' => $d['base_currency'],
      '#attached' => [
        'library' => ['currencyapi/currency_rates'],
      ],
    ];
  }

  /**
   * Json output.
   */
  public function ratesJson($key): JsonResponse {

    // @todo add sopme basic validation on $key.
    $d = [];
    $d['service'] = \Drupal::service('currencyapi.service');
    $d['storage'] = $d['service']->getAllRates();

    return new JsonResponse($d['storage']);
  }

}
